#!/usr/bin/perl

########################################################
#
# CMS - Content Management System
# http://sourceforge.net/projects/cms
# created on June 13th 2000
# by Alexandre Gravel (gravel@infivia.com)
#
# main program file
#
#
########################################################


use CGI;
use URI::Escape;
use cmstools;
use cmsconfig;

$cgi = CGI::new();
$script = $cgi->url;

# If we need to set a cookie
if ($cgi->param('username') && ($cgi->param('username') ne 'email address') ) {
	$username = $cgi->param('username');
	$thecookie = $cgi->cookie(-name=>'username',-value=>"$username",-expires=>'+3M');
	print $cgi->header(-cookie=>$thecookie);
}
elsif ($cgi->param('logout')) {
	$thecookie = $cgi->cookie(-name=>'username',-value=>'nobody',-expires=>'now');
	print $cgi->header(-cookie=>$thecookie);
}
else {
	print $cgi->header();
}

# If we want to login
if ($cgi->param('login')) {
	printHeaderLogin("Log-in");
	Login();
	printFooterPlain();
}
# If we just logged in
elsif ($cgi->param('username')) {
	printHeaderLogin("Logged-in");
	LoggedIn();
	printFooterPlain();
}

# If we want to logout
elsif ($cgi->param('logout')) {
	printHeaderLogin("Logged-out");
	Logout();
	printFooterPlain();
}

# Else we do business as usual
else {
	$username = $cgi->cookie('username');
	if ($cgi->param('currentdir')) {
		$currentdir = $contentdir;
		$currentdir .= uri_unescape($cgi->param('currentdir'));
	}
	else {
		$currentdir = $contentdir . "/";
	}
	
	if ($cgi->param('changedir')) {
		$changedir = uri_unescape($cgi->param('changedir'));
		if ($changedir eq ".." && $currentdir ne $contentdir) {
	
			# Move one directory up
			$numberofdirs = split(/\//,$currentdir);
			@dirlist = split(/\//,$currentdir);
			$currentdir = "";
			for ($i = 1; $i < ($numberofdirs-1); $i++) {
				$currentdir .= "/" . $dirlist[$i];
			}
			$currentdir .= "/";
	
		}
		elsif ($changedir ne "." && $changedir ne "..") {
			$currentdir = $currentdir . $changedir . "/";
		}
	}
	
	# If security is at risk, quit
	if ( ($currentdir =~ /\.\./) || !($currentdir =~ /$contentdir/) ) {
		print("Security is at risk : $currentdir");
		die;
	}

	@files = SetDir($currentdir);
	
	%reservedfiles = GetReservedFiles(@files);
	%reservedwhen = GetReservedWhen(@files);
	%comments = GetComments(@files);
	%reasons = GetReasons(@files);
	%lastmodifiedby = GetLastModifiedBy(@files);
	%deletedfiles = GetDeletedFiles(@files);
	
	#Let's convert the currentdir to a nicer string
	$displaydir = $currentdir;
	$displaydir =~ s/$contentdir//;
	$escdisplaydir = uri_escape($displaydir);
	
	if ($cgi->param('action') eq "newfile") {
		if (!$cgi->param('newfilecontent')) {
			printHeaderFile("new file in $displaydir");
			NewFileForm();
		}
		else {
			$file = uri_unescape($cgi->param('file'));
			printHeaderFile("$displaydir$file added");
			NewFile();
		}			
	}
	elsif ($cgi->param('action') eq "newdirectory") {
		if (!$cgi->param('newdirectory')) {
			printHeaderFile("new directory in $displaydir");
			NewDirectoryForm();
		}
		else {
			$newdirectory = uri_unescape($cgi->param('newdirectory'));
			printHeaderFile("$displaydir$newdirectory created");
			NewDirectory();
		}			
	}
	elsif ($cgi->param('action') eq "comment") {
		$file = uri_unescape($cgi->param('file'));
		if (!$cgi->param('comment')) {
			printHeaderFile("new comment for $file");
			CommentForm();
		}
		else {
			printHeaderFile("comment set for $file");
			Comment();
		}

	}
	elsif (!$cgi->param('file')) {
		printHeader("CMS - $displaydir");
		DisplayDir();
	}
	else {
		$file = uri_unescape($cgi->param('file'));
		if (!$cgi->param('action')) {
			printHeaderFile("$file");
			DisplayFile();
		}
		elsif($cgi->param('action') eq 'checkout') {
			if (!$cgi->param('reserved')) {
				printHeaderFile("checking-out $file");
				CheckOutForm();
			}
			else {
				printHeaderCheckOut("checked-out $file");
				CheckOut();
			}
		}
		elsif($cgi->param('action') eq 'checkin') {
			if (!$cgi->param('filecontent')) {
				printHeaderFile("checking-in $file");
				CheckInForm();
			}
			else {
				printHeaderFile("checked-in $file");
				CheckIn();
			}
		}
		elsif($cgi->param('action') eq 'discardreservation') {
			if (!$cgi->param('confirmation')) {
				printHeaderFile("discard reservation confirmation for $file");
				DiscardReservationForm();
			}
			else {
				printHeaderFile("discarded reservation for $file");
				DiscardReservation();
			}
		}
		elsif($cgi->param('action') eq 'delete') {
			if (!$cgi->param('confirmation')) {
				printHeaderFile("delete confirmation for $file");
				DeleteForm();
			}
			else {
				printHeaderFile("deleted $file");
				Delete();
			}
		}
	}

	printFooter();

}


sub Login {

	print "<TABLE CELLSPACING=0 WIDTH=100% BGCOLOR=#CCCCCC CELLPADDING=5 BORDER=0><TR><TD VALIGN=TOP>";
	print "<H3>";
	printIcon($file);
	print "Please identify yourself (email address)</H3>";
	print "</TD></TR></TABLE>";


	print "<TABLE CELLSPACING=0 CELLPADDING=10 WIDTH=100% BORDER=0><TR><TD VALIGN=TOP>";

	print $cgi->startform(-name=>'theform');

	print $cgi->textfield(-name=>"username",-value=>"email address");

	print qq~
		<SCRIPT LANGUAGE="javascript">

		<!--
			document.theform.username.focus();
		//-->

		</SCRIPT>
		~;

	print $cgi->submit();

	print "</TD></TR></TABLE>";

}

sub LoggedIn {

	$username = $cgi->cookie('username');

	# If the user did not enter a username or a valid email address
	if (!$username || !($username =~ /\@/) ) {
		Login();
		return;
	}

	print "<TABLE CELLSPACING=0 WIDTH=100% BGCOLOR=#CCCCCC CELLPADDING=5 BORDER=0><TR><TD VALIGN=TOP>";
	print "<H3>";
	printIcon($file);
	print "Logged-in as $username</H3>";
	print "</TD></TR></TABLE>";


	print "<TABLE CELLSPACING=0 CELLPADDING=10 WIDTH=100% BORDER=0><TR><TD VALIGN=TOP>";

	print qq~<A HREF="javascript:window.close()">close window</A>~;

	print "</TD></TR></TABLE>";

}

sub Logout {

	print "<TABLE CELLSPACING=0 WIDTH=100% BGCOLOR=#CCCCCC CELLPADDING=5 BORDER=0><TR><TD VALIGN=TOP>";
	print "<H3>";
	printIcon($file);
	print "You are now logged out</H3>";
	print "</TD></TR></TABLE>";


	$cgi->cookie(-name=>'username',-value=>"",-expires=>"now");

	print "<TABLE CELLSPACING=0 CELLPADDING=10 WIDTH=100% BORDER=0><TR><TD VALIGN=TOP>";

	print qq~<A HREF="javascript:window.close()">close window</A>~;

	print "</TD></TR></TABLE>";

}

sub SetDir {

	my $dir = shift;

	if ( !(opendir(DIR,$dir)) ) {
		return false;
	}

	chdir($dir);

	my @files = readdir(DIR);

	closedir(DIR);

	return @files;

}

sub GetReservedFiles {

	my %reservedfiles;

	foreach my $file (@files) {

		if ( ($file =~ /(.*)\.reserved$/) ) {
			$filename = $1;
			open(FILE,$file);
			$reserveid = <FILE>;
			close(FILE);
			$reservedfiles{$filename} = $reserveid;
			$controlfiles{$file} = $filename;
		}

	}

	return %reservedfiles;

}

sub GetReservedWhen {

	my %reservedwhen;

	foreach my $file (@files) {

		if ( ($file =~ /(.*)\.reservedwhen$/) ) {
			$filename = $1;
			open(FILE,$file);
			$when = <FILE>;
			close(FILE);
			$reservedwhen{$filename} = $when;
			$controlfiles{$file} = $filename;
		}

	}

	return %reservedwhen;

}

sub GetComments {

	my %comments;

	foreach my $file (@files) {

		if ( ($file =~ /(.*)\.comment$/) ) {
			$filename = $1;
			open(FILE,$file);
			$comment = <FILE>;
			close(FILE);
			$comments{$filename} = $comment;
			$controlfiles{$file} = $filename;
		}

	}

	return %comments;

}

sub GetReasons {

	my %reasons;

	foreach my $file (@files) {

		if ( ($file =~ /(.*)\.reason$/) ) {
			$filename = $1;
			open(FILE,$file);
			$reason = <FILE>;
			close(FILE);
			$reasons{$filename} = $reason;
			$controlfiles{$file} = $filename;
		}

	}

	return %reasons;

}

sub GetDeletedFiles {

	my %deletedfiles;

	foreach my $file (@files) {

		if ( ($file =~ /(.*)\.deleted$/) ) {
			$filename = $1;
			$deletedfiles{$filename} = $file;
			$controlfiles{$file} = $filename;
		}

	}

	return %deletedfiles;

}

sub GetLastModifiedBy {

	my %lastmodifiedby;

	foreach my $file (@files) {

		if ( ($file =~ /(.*)\.last$/) ) {
			$filename = $1;
			open(FILE,$file);
			$lastname = <FILE>;
			close(FILE);
			$lastmodifiedby{$filename} = $lastname;
			$controlfiles{$file} = $filename;
		}

	}

	return %lastmodifiedby;

}

sub DisplayDir {

	# Print the top of the page
	print "<TABLE CELLSPACING=0 WIDTH=100% CELLPADDING=5 BORDER=0><TR><TD VALIGN=TOP>";
	print "<H3>&nbsp current directory: $displaydir</H3>";
	print "</TD></TR></TABLE>";

 	# Print the create file option
	print qq~&nbsp <A HREF="javascript:newfile()"><img align=absmiddle border=0 width=25 height=25 src="$graphurl/file_white.gif"> new file </A>~;

 	# Print the create directory option
	print qq~&nbsp&nbsp <A HREF="javascript:newdirectory()"><img align=absmiddle border=0 width=25 height=25 src="$graphurl/folder.gif"> new directory </A>~;

	print "<BR><HR SIZE=1 WIDTH=100%>";

	# Print the table header
	$trcolor = trcolor($trcolor);
	print qq~
		<TABLE CELLSPACING=0 BORDER=0 WIDTH=100%>
			<TR $trcolor>
				<TD VALIGN=TOP>&nbsp</TD>
				<TD VALIGN=TOP><B>filename</B></TD>
				<TD VALIGN=TOP><B>size</B></TD>
				<TD VALIGN=TOP><B>date</B></TD>
				<TD VALIGN=TOP><B>status</B></TD>
				<TD VALIGN=TOP><B>comment</B></TD>
			</TR>
		~;

	# Let's first display all the subdirectories
	foreach my $file (@files) {

		# If it is a directory and not the current pointer
		if (-d $file && $file ne ".") {

			# If we are not at the top and this is the parent pointer
			if ( ($file ne "..") || ($file eq ".." && $displaydir ne "/")) {

				$trcolor = trcolor($trcolor);
				print "<TR $trcolor>";
	
				# We print the icon
				if ($file eq "..") {
					print qq~<TD VALIGN=TOP><img align=absmiddle border=0 width=25 height=25 src="$graphurl/back.gif"></TD>~;
				}
				else {
					print qq~<TD VALIGN=TOP><img align=absmiddle border=0 width=25 height=25 src="$graphurl/folder.gif"></TD>~;
				}
	
				# We print the filename
				#If it isn't reserved, we print the link
				print "<TD VALIGN=TOP>";
				if ($reservedfiles{$file}) {
					print qq~<FONT COLOR=#FF0000>~;
				}
				else {
					$escfile = uri_escape($file);
					print qq~<A HREF="$script?currentdir=$escdisplaydir&changedir=$escfile">~;
				}
	
				if ($file eq "..") {
					print " (parent directory)";
				}
				else {
					print "$file/";
				}
	
				if ($reservedfiles{$file}) {
					print "</FONT>";
				}
	
				print "</A></TD>";
	
				# We print the size
				$sizekb = int((stat($file))[7]/1000);
				print "<TD VALIGN=TOP>$sizekb kB</TD>";
	
				# We print the date
				$mtime = (stat($file))[9];
				$date = localtime($mtime);
				print "<TD VALIGN=TOP>$date</TD>";
	
				# We print the status
				print "<TD VALIGN=TOP>&nbsp</TD>";
	
				# We print the comment
				print "<TD VALIGN=TOP>";
				if ($file ne "." && $file ne "..") {
					print qq~<A HREF="javascript:comment('$file')">~;
					print qq~<img align=absmiddle border=0 width=10 height=10 src="$graphurl/comment.gif">~;
					print "</A>";
				}
					print "&nbsp $comments{$file}</TD>";
				print "</TR>";
			}

		}

	}

	# Let's now display all the files in this directory
	foreach my $file (@files) {

		# If it is a directory and not a control file
		if (-f $file && !$controlfiles{$file}) {
			$trcolor = trcolor($trcolor);
			print "<TR $trcolor>";

			# We print the icon
			print "<TD VALIGN=TOP>";
			printIcon($file);
			print "</TD>";

			# We print the filename
			print "<TD VALIGN=TOP>";
			print qq~<A HREF="javascript:fileInfo('$file')">~;
			if ($reservedfiles{$file} eq $username && $username ne "") {
				print qq~<FONT COLOR=#009900>~;
			}

			elsif ($reservedfiles{$file}) {
				print qq~<FONT COLOR=#FF0000>~;
			}

			print "$file";

			if ($reservedfiles{$file}) {
				print "</FONT>";
			}

			print "</A></TD>";

			# We print the size
			$sizekb = int((stat($file))[7]/1000);
			print "<TD VALIGN=TOP>$sizekb kB</TD>";

			# We print the date
			$mtime = (stat($file))[9];
			$date = localtime($mtime);
			print "<TD VALIGN=TOP>$date</TD>";

			# We get the reservation reason if there is one and generate a reason string
			if ($reasons{$file}) {
				$thereason = $reasons{$file};
				$reason = "<BR>reason: $thereason";
			}

			# We print the status
			if ($reservedfiles{$file} eq $username && $username ne "") {
				$reserveid = $reservedfiles{$file};
				print "<TD VALIGN=TOP><FONT COLOR=#009900>reserved by $reserveid$reason</FONT></TD>";
			}
			elsif ($reservedfiles{$file}) {
				$reserveid = $reservedfiles{$file};
				print "<TD VALIGN=TOP><FONT COLOR=#FF0000>reserved by $reserveid$reason</FONT></TD>";
			}
			else {
				print "<TD VALIGN=TOP><FONT COLOR=#0000FF>available</FONT></TD>";
			}

			# We print the comment
			print "<TD VALIGN=TOP>";
			print qq~<A HREF="javascript:comment('$file')">~;
			print qq~<img align=absmiddle border=0 width=10 height=10 src="$graphurl/comment.gif">~;
			print "</A>&nbsp $comments{$file}</TD>";

			print "</TR>";

		}

	}

	# Print the table footer
	print "</TABLE>";

	print "<HR SIZE=1 WIDTH=100%>";

	print qq~
		<BR>
		<img align=absmiddle border=0 width=25 height=25 src="$graphurl/file_green.gif"><FONT COLOR=#09900>files currently reserved by you</FONT> 
		<img align=absmiddle border=0 width=25 height=25 src="$graphurl/file_red.gif"><FONT COLOR=#FF0000>files reserved by someone else</FONT> 
		<img align=absmiddle border=0 width=25 height=25 src="$graphurl/file_blue.gif"><FONT COLOR=#0000FF>files not reserved</FONT> 
		<BR>
	~;

}


sub DisplayFile {

	# Print the top of the page
	print "<TABLE CELLSPACING=0 WIDTH=100% BGCOLOR=#CCCCCC CELLPADDING=5 BORDER=0><TR><TD VALIGN=TOP>";
	print "<H3>";
	printIcon($file);
	print "$file";

	# We print the comment
	if ($comments{$file}) {
		print "<BR><I>($comments{$file})</I>";
	}

	print "</H3>";
	print "</TD></TR></TABLE>";

	print "<TABLE CELLSPACING=0 CELLPADDING=10 WIDTH=100% BORDER=0><TR><TD VALIGN=TOP>";

	# We print the size
	$sizekb = int((stat($file))[7]/1000);
	print "$sizekb kB";

	# We print the date
	$mtime = (stat($file))[9];
	$date = localtime($mtime);
	print " ($date)<BR>";

	# We get the reservation reason if there is one and generate a reason string
	if ($reasons{$file}) {
		$thereason = $reasons{$file};
		$reason = "<BR>reason: $thereason";
	}

	# We get the reservation time if there is one and generate a reservationtime string
	if ($reservedwhen{$file}) {
		$thetime = $reservedwhen{$file};
		$reservationtime = "<BR>when: $thetime";
	}

	# We print the status
	if ($reservedfiles{$file}) {
		$reserveid = $reservedfiles{$file};
		if ($reserveid eq $username) {
			print "<FONT COLOR=#009900>";
		}
		else {
			print "<FONT COLOR=#FF0000>";
		}
		print "reserved by $reserveid$reservationtime$reason</FONT><BR>";
	}
	else {
		$lastname = $lastmodifiedby{$file};
		print "<FONT COLOR=#0000FF>available</FONT><BR>";
		if ($lastname) {
			print "last modifications by $lastname<BR>";
		}
	}

	print "<HR SIZE=1 WIDTH=100%>";

	# Check-out
	print qq~<A HREF="javascript:checkout()"><img align=absmiddle border=0 width=25 height=25 src="$graphurl/file_out.gif"> check-out </A><BR>~;

	# Check-in
	print qq~<A HREF="javascript:checkin()"><img align=absmiddle border=0 width=25 height=25 src="$graphurl/file_in.gif"> check-in </A><BR>~;

	# Discard reservation
	if ($reservedfiles{$file}) {
		print qq~<A HREF="javascript:discardreservation()"><img align=absmiddle border=0 width=25 height=25 src="$graphurl/file_x.gif"> discard reservation </A><BR>~;
	}

	# Delete
	print qq~<A HREF="javascript:deletefile()"><img align=absmiddle border=0 width=25 height=25 src="$graphurl/file_xred.gif"> delete the file </A><BR>~;

	# Comment
	print qq~<A HREF="javascript:comment()"><img align=absmiddle border=0 width=25 height=25 src="$graphurl/comment.gif"> change comment </A><BR>~;

	# Close window
	print qq~<BR><A HREF="javascript:window.close()">close window</A><BR>~;

	print "</TD></TR></TABLE>";

}


sub CheckOutForm {

	print "<TABLE CELLSPACING=0 WIDTH=100% BGCOLOR=#CCCCCC CELLPADDING=5 BORDER=0><TR><TD VALIGN=TOP>";
	print "<H3>";
	printIcon($file);
	print "checking-out $file</H3>";
	print "</TD></TR></TABLE>";


	print "<TABLE CELLSPACING=0 CELLPADDING=10 WIDTH=100% BORDER=0><TR><TD VALIGN=TOP>";

	if ( $reservedfiles{$file} ) {
		print "This file is currently reserved, you can only get it unreserved";
		print "<TABLE CELLSPACING=10 CELLPADDING=10 WIDTH=100% BORDER=0><TR><TD VALIGN=TOP BGCOLOR=lightyellow WIDTH=50%>";
	}
	else {
		print "<CENTER><TABLE CELLSPACING=10 CELLPADDING=10 BORDER=0 WIDTH=100%><TR><TD VALIGN=TOP BGCOLOR=lightyellow WIDTH=50%><CENTER>";
		print "If you intend to modify it<BR>";
		print $cgi->startform(-name=>"reserved");
		print $cgi->hidden(-name=>"currentdir",-value=>"$displaydir");
		print $cgi->hidden(-name=>"file",-value=>"$file");
		print $cgi->hidden(-name=>"action",-value=>"checkout");
		print $cgi->hidden(-name=>"reserved",-value=>"yes");
		print $cgi->textfield(-name=>"reason",-value=>"enter reason");
		print "<BR>";
		print $cgi->submit(-label=>"reserved");
		print "</form></CENTER></TD>";
	}

	print "<TD VALIGN=TOP BGCOLOR=lightyellow WIDTH=50%><CENTER>";
	print "If you do not intend to modify it<BR><BR>";
	print $cgi->startform(-name=>"unreserved");
	print $cgi->hidden(-name=>"currentdir",-value=>"$displaydir");
	print $cgi->hidden(-name=>"file",-value=>"$file");
	print $cgi->hidden(-name=>"action",-value=>"checkout");
	print $cgi->hidden(-name=>"reserved",-value=>"no");
	print $cgi->submit(-label=>"unreserved");
	print "</form></CENTER></TD></TR></TABLE></CENTER>";

	# Cancel
	print qq~<A HREF="javascript:window.close()">cancel</A><BR>~;


	print "</TD></TR></TABLE>";

}


sub CheckOut {

	print "<TABLE CELLSPACING=0 WIDTH=100% BGCOLOR=#CCCCCC CELLPADDING=5 BORDER=0><TR><TD VALIGN=TOP>";
	print "<H3>";
	printIcon($file);
	print "checked-out $file</H3>";
	print "</TD></TR></TABLE>";


	print "<TABLE CELLSPACING=0 CELLPADDING=10 WIDTH=100% BORDER=0><TR><TD VALIGN=TOP>";

	# If it was asked reserved, we reserve it
	if ($cgi->param('reserved') eq "yes" && $username) {

		open (LOCK,">$file.reserved");
		print LOCK "$username";
		close (LOCK);

		$when = localtime();
		open (WHEN,">$file.reservedwhen");
		print WHEN $when;
		close (WHEN);

		print "<H3>now reserved by $username</H3>";

		Log("[$username] RESERVED $currentdir$file");

	}

	# If there was a reason, we store it.
	if ($cgi->param('reason') ne "") {

		$reason = uri_unescape($cgi->param('reason'));
		open (LOCK,">$file.reason");
		print LOCK "$reason";
		close (LOCK);

		print "<H3>$reason</H3>";

		Log("[$username] REASON $currentdir$file");

	}


	# File is sent by Javascript
	print qq~<H4>the file will download automatically</H4>~;

	Log("[$username] SENT $currentdir$file");

	# Close window
	print qq~<A HREF="javascript:window.close()">close window</A><BR>~;

	print "</TD></TR></TABLE>";

}


sub CheckInForm {

	print "<TABLE CELLSPACING=0 WIDTH=100% BGCOLOR=#CCCCCC CELLPADDING=5 BORDER=0><TR><TD VALIGN=TOP>";
	print "<H3>";
	printIcon($file);
	print "checking-in $file</H3>";
	print "</TD></TR></TABLE>";


	print "<TABLE CELLSPACING=0 CELLPADDING=10 WIDTH=100% BORDER=0><TR><TD VALIGN=TOP>";

	if ( $reservedfiles{$file} && $reservedfiles{$file} ne $username ) {
		$thereserver = $reservedfiles{$file};
		print "This file is currently reserved by $thereserver, you cannot check-in";
	}
	else{
		if ( $reservedfiles{$file} eq $username && $username ne "" ) {
			print "This file is currently reserved by you, you can check-in";
		}
		else {
			print "This file is currently available, you can check-in";
		}

		print $cgi->start_multipart_form(-name=>"checkin");
		print $cgi->hidden(-name=>"currentdir",-value=>"$displaydir");
		print $cgi->hidden(-name=>"file",-value=>"$file");
		print $cgi->hidden(-name=>"action",-value=>"checkin");
		print $cgi->filefield(-name=>"filecontent");
		print $cgi->submit(-label=>"check-in");
	}

	# Cancel
	print qq~<BR><BR><A HREF="javascript:window.close()">cancel</A><BR>~;

	print "</TD></TR></TABLE>";

}

sub CheckIn {

	print "<TABLE CELLSPACING=0 WIDTH=100% BGCOLOR=#CCCCCC CELLPADDING=5 BORDER=0><TR><TD VALIGN=TOP>";
	print "<H3>";
	printIcon($file);
	print "checked-in $file</H3>";
	print "</TD></TR></TABLE>";


	print "<TABLE CELLSPACING=0 CELLPADDING=10 WIDTH=100% BORDER=0><TR><TD VALIGN=TOP>";

	# If it was reserved, we unreserve it
	if ($reservedfiles{$file}) {

		rename "$file.reserved","$file.last";
		rename "$file.reservedwhen","$file.reservedwhen.last";

		Log("[$username] UNRESERVED $currentdir$file");

		print "<H3>last modifications done by $username</H3>";

	}

	# If it had a reason file, we delete it
	if ($reasons{$file}) {

		rename "$file.reason","$file.reason.deleted";

		Log("[$username] REASONDELETED $currentdir$file");

	}

	# Write the file...
	$filecontent = $cgi->param('filecontent');

	open(FILE,">$file");
	while ($bytesread = read($filecontent,$buffer,1024)) {
		print FILE $buffer;
	}
	close FILE;

	Log("[$username] WROTE $currentdir$file");

	# Close window
	print qq~<BR><BR><A HREF="javascript:window.close()">close window</A><BR>~;

	print "</TD></TR></TABLE>";

}

sub CommentForm {

	print "<TABLE CELLSPACING=0 WIDTH=100% BGCOLOR=#CCCCCC CELLPADDING=5 BORDER=0><TR><TD VALIGN=TOP>";
	print "<H3>";
	printIcon($file);
	print "new comment for $file</H3>";
	print "</TD></TR></TABLE>";


	print "<TABLE CELLSPACING=0 CELLPADDING=10 WIDTH=100% BORDER=0><TR><TD VALIGN=TOP>";

	$previouscomment = $comments{$file};

	print $cgi->startform(-name=>"comment");
	print $cgi->hidden(-name=>"currentdir",-value=>"$displaydir");
	print $cgi->hidden(-name=>"file",-value=>"$file");
	print $cgi->hidden(-name=>"action",-value=>"comment");
	print $cgi->textfield(-name=>"comment",-value=>"$previouscomment");
	print $cgi->submit(-label=>"set comment");

	# Cancel
	print qq~<BR><BR><A HREF="javascript:window.close()">cancel</A><BR>~;

	print "</TD></TR></TABLE>";

}

sub Comment {

	print "<TABLE CELLSPACING=0 WIDTH=100% BGCOLOR=#CCCCCC CELLPADDING=5 BORDER=0><TR><TD VALIGN=TOP>";
	print "<H3>";
	printIcon($file);
	print "comment set for $file</H3>";
	print "</TD></TR></TABLE>";


	print "<TABLE CELLSPACING=0 CELLPADDING=10 WIDTH=100% BORDER=0><TR><TD VALIGN=TOP>";

	# Write the file...
	$comment = uri_unescape($cgi->param('comment'));
	open(FILE,">$file.comment");
	print FILE $comment;
	close FILE;

	print "<H3>$comment</H3>";

	Log("[$username] COMMENT $currentdir$file");

	# Close window
	print qq~<A HREF="javascript:window.close()">close window</A><BR>~;

	print "</TD></TR></TABLE>";

}

sub NewFileForm {

	print "<TABLE CELLSPACING=0 WIDTH=100% BGCOLOR=#CCCCCC CELLPADDING=5 BORDER=0><TR><TD VALIGN=TOP>";
	print "<H3>";
	printIcon($file);
	print "new file in $displaydir</H3>";
	print "</TD></TR></TABLE>";


	print "<TABLE CELLSPACING=0 CELLPADDING=10 WIDTH=100% BORDER=0><TR><TD VALIGN=TOP>";

	print $cgi->start_multipart_form(-name=>"newfile");
	print $cgi->hidden(-name=>"currentdir",-value=>"$displaydir");
	print $cgi->hidden(-name=>"action",-value=>"newfile");
	print $cgi->filefield(-name=>"newfilecontent");
	print $cgi->submit(-label=>"upload file");

	# Cancel
	print qq~<BR><BR><A HREF="javascript:window.close()">cancel</A><BR>~;

	print "</TD></TR></TABLE>";

}

sub NewFile {

	$filecontent = $cgi->param('newfilecontent');

	if ($filecontent =~ /([^\/\\]+)$/) {
		$file="$1";
	}
	else {
		$file="$filepath";
	}

	print "<TABLE CELLSPACING=0 WIDTH=100% BGCOLOR=#CCCCCC CELLPADDING=5 BORDER=0><TR><TD VALIGN=TOP>";
	print "<H3>$displaydir$file added</H3>";
	print "</TD></TR></TABLE>";


	print "<TABLE CELLSPACING=0 CELLPADDING=10 WIDTH=100% BORDER=0><TR><TD VALIGN=TOP>";

	# Write the file...
	open(FILE,">$file");
	while ($bytesread = read($filecontent,$buffer,1024)) {
		print FILE $buffer;
	}
	close FILE;

	Log("[$username] WROTE $currentdir$file");

	# Close window
	print qq~<BR><BR><A HREF="javascript:window.close()">close window</A><BR>~;

	print "</TD></TR></TABLE>";

}

sub NewDirectoryForm {

	print "<TABLE CELLSPACING=0 WIDTH=100% BGCOLOR=#CCCCCC CELLPADDING=5 BORDER=0><TR><TD VALIGN=TOP>";
	print "<H3>";
	printIcon($file);
	print "new directory in $displaydir</H3>";
	print "</TD></TR></TABLE>";


	print "<TABLE CELLSPACING=0 CELLPADDING=10 WIDTH=100% BORDER=0><TR><TD VALIGN=TOP>";

	print $cgi->startform(-name=>"newdirectory");
	print $cgi->hidden(-name=>"currentdir",-value=>"$displaydir");
	print $cgi->hidden(-name=>"action",-value=>"newdirectory");
	print $cgi->textfield(-name=>"newdirectory",-value=>"directory name here");
	print $cgi->submit(-label=>"create directory");

	# Cancel
	print qq~<BR><BR><A HREF="javascript:window.close()">cancel</A><BR>~;

	print "</TD></TR></TABLE>";

}

sub NewDirectory {

	print "<TABLE CELLSPACING=0 WIDTH=100% BGCOLOR=#CCCCCC CELLPADDING=5 BORDER=0><TR><TD VALIGN=TOP>";
	print "<H3>";
	printIcon($file);
	print "$displaydir$newdirectory added</H3>";
	print "</TD></TR></TABLE>";


	print "<TABLE CELLSPACING=0 CELLPADDING=10 WIDTH=100% BORDER=0><TR><TD VALIGN=TOP>";

	# Create the directory...
	mkdir($newdirectory,0777);
	chmod(0777,$newdirectory);

	Log("[$username] NEWDIRECTORY $currentdir$newdirectory");

	# Close window
	print qq~<BR><BR><A HREF="javascript:window.close()">close window</A><BR>~;

	print "</TD></TR></TABLE>";

}

sub DiscardReservationForm {

	print "<TABLE CELLSPACING=0 WIDTH=100% BGCOLOR=#CCCCCC CELLPADDING=5 BORDER=0><TR><TD VALIGN=TOP>";
	print "<H3>";
	printIcon($file);
	print "discard reservation confirmation for $file</H3>";
	print "</TD></TR></TABLE>";


	print "<TABLE CELLSPACING=0 CELLPADDING=10 WIDTH=100% BORDER=0><TR><TD VALIGN=TOP>";

	if ( $reservedfiles{$file} && $reservedfiles{$file} ne $username ) {
		print "This file is currently reserved by $username, you cannot discard the reservation";
	}
	elsif ( $reservedfiles{$file} eq $username && $username ne "" ) {
		print "This file is currently reserved by you, you can discard the reservation";
	
		print $cgi->start_multipart_form(-name=>"discardreservation");
		print $cgi->hidden(-name=>"currentdir",-value=>"$displaydir");
		print $cgi->hidden(-name=>"file",-value=>"$file");
		print $cgi->hidden(-name=>"action",-value=>"discardreservation");
		print $cgi->hidden(-name=>"confirmation",-value=>"yes");
		print $cgi->submit(-label=>"confirm discard reservation");
	}

	# Cancel
	print qq~<BR><BR><A HREF="javascript:window.close()">cancel</A><BR>~;

	print "</TD></TR></TABLE>";

}

sub DiscardReservation {

	print "<TABLE CELLSPACING=0 WIDTH=100% BGCOLOR=#CCCCCC CELLPADDING=5 BORDER=0><TR><TD VALIGN=TOP>";
	print "<H3>";
	printIcon($file);
	print "discard reservation for $file</H3>";
	print "</TD></TR></TABLE>";


	print "<TABLE CELLSPACING=0 CELLPADDING=10 WIDTH=100% BORDER=0><TR><TD VALIGN=TOP>";

	unlink("$file.reserved");
	unlink("$file.reservedwhen");

	Log("[$username] DISCARDRESERVATION $currentdir$file");

	# If it had a reason file, we delete it
	if ($reasons{$file}) {

		rename "$file.reason","$file.reason.deleted";

		Log("[$username] REASONDELETED $currentdir$file");

	}

	print "The reservation was discarded.";

	# Close window
	print qq~<BR><BR><A HREF="javascript:window.close()">close window</A><BR>~;

	print "</TD></TR></TABLE>";

}

sub DeleteForm {

	print "<TABLE CELLSPACING=0 WIDTH=100% BGCOLOR=#CCCCCC CELLPADDING=5 BORDER=0><TR><TD VALIGN=TOP>";
	print "<H3>";
	printIcon($file);
	print "delete confirmation for $file</H3>";
	print "</TD></TR></TABLE>";


	print "<TABLE CELLSPACING=0 CELLPADDING=10 WIDTH=100% BORDER=0><TR><TD VALIGN=TOP>";

	if ( $reservedfiles{$file} && $reservedfiles{$file} ne $username ) {
		print "This file is currently reserved by $username, you cannot delete it";
	}
	else{
		if ( $reservedfiles{$file} eq $username && $username ne "" ) {
			print "This file is currently reserved by you, you can delete it";
		}
		else {
			print "This file is currently available, you can delete it";
		}

		print $cgi->start_multipart_form(-name=>"delete");
		print $cgi->hidden(-name=>"currentdir",-value=>"$displaydir");
		print $cgi->hidden(-name=>"file",-value=>"$file");
		print $cgi->hidden(-name=>"action",-value=>"delete");
		print $cgi->hidden(-name=>"confirmation",-value=>"yes");
		print $cgi->submit(-label=>"confirm delete");
	}

	# Cancel
	print qq~<BR><BR><A HREF="javascript:window.close()">cancel</A><BR>~;

	print "</TD></TR></TABLE>";

}

sub Delete {

	print "<TABLE CELLSPACING=0 WIDTH=100% BGCOLOR=#CCCCCC CELLPADDING=5 BORDER=0><TR><TD VALIGN=TOP>";
	print "<H3>";
	printIcon($file);
	print "deleted $file</H3>";
	print "</TD></TR></TABLE>";


	print "<TABLE CELLSPACING=0 CELLPADDING=10 WIDTH=100% BORDER=0><TR><TD VALIGN=TOP>";

	# If it had a last modification file, we delete it
	if ($lastmodifiedby{$file}) {

		rename "$file.last","$file.last.deleted";

		Log("[$username] LASTMODIFICATIONDELETED $currentdir$file");

	}

	# If it was reserved, we unreserve it
	if ($reservedfiles{$file}) {

		rename "$file.reserved","$file.last.deleted";
		rename "$file.reservedwhen","$file.reservedwhen.last.deleted";

		Log("[$username] UNRESERVEDDELETED $currentdir$file");

	}

	# If it had a reason file, we delete it
	if ($reasons{$file}) {

		rename "$file.reason","$file.reason.deleted";

		Log("[$username] REASONDELETED $currentdir$file");

	}

	# If it had a comment file, we delete it
	if ($comments{$file}) {

		rename "$file.comment","$file.comment.deleted";

		Log("[$username] COMMENTDELETED $currentdir$file");

	}

	# Delete the file...
	rename "$file","$file.deleted";

	Log("[$username] DELETED $currentdir$file");

	# Close window
	print qq~<BR><BR><A HREF="javascript:window.close()">close window</A><BR>~;

	print "</TD></TR></TABLE>";

}


sub printIcon {

	my $file = shift;

	if (!$file) {
		print qq~<img align=absmiddle border=0 width=25 height=25 src="$graphurl/file_white.gif">~;
		return;
	}

	if ($reservedfiles{$file} eq $username && $username ne "") {
		print qq~<img align=absmiddle border=0 width=25 height=25 src="$graphurl/file_green.gif">~;
	}
	elsif ($reservedfiles{$file}) {
		print qq~<img align=absmiddle border=0 width=25 height=25 src="$graphurl/file_red.gif">~;
	}
	else {
		print qq~<img align=absmiddle border=0 width=25 height=25 src="$graphurl/file_blue.gif">~;
	}
}
